// [[Rcpp::interfaces]]

#include <Rcpp.h>
using namespace Rcpp;

#define rownames(x) as<CharacterVector>(VECTOR_ELT(x.attr("dimnames"), 0))
#define colnames(x) as<CharacterVector>(VECTOR_ELT(x.attr("dimnames"), 1))

// x is a list of logical matrices, corresponding to 'expressed' or
// 'unexpressed'
// with rows as cells and columns as cytokines

// combos is a list of integer vectors as follows:
// for each cytokine we wish to check, we pass its index (1, 2, ...)
// if we want to check if it is positive, we pass a positive number
// if we want to check if it is negative, we pass a negative number, of the
// same index

// eg: for a matrix with columns CD4 and TNFa, we might pass combos=list(c(1,
// -2))
// to get counts that are CD4+TNFa-

// [[Rcpp::export(.CellCounts)]]
IntegerMatrix CellCounts(List x, List combos) {

  int x_n = x.size();
  int combos_n = combos.size();
  IntegerMatrix output(x_n, combos_n);

  for (int i = 0; i < x_n; ++i) {
    // Rcout << "Working with matrix " << i+1 << " of " << x_n << std::endl;
    LogicalMatrix mat = as<LogicalMatrix>(x[i]);
    int nrows = mat.nrow();
    for (int k = 0; k < combos_n; ++k) {

      int num = 0;
      IntegerVector c_combo = as<IntegerVector>(combos[k]);
      int n_c = c_combo.size();

      IntegerVector c_combo_abs = sapply(c_combo, ::abs);

      for (int j = 0; j < nrows; ++j) {

        LogicalMatrix::Row row = mat(j, _);

// checking algorithm:
// we loop through each entry in 'c_combo' => p
// we check the element in 'row' at 'abs(c_combo[p])'
// if 'abs(c_combo[p])' is > 0 and row[ c_combo[p] ] is true; continue
// if 'abs(c_combo[p])' is <= 0 and row[ c_combo[p] ] is false; continue
// else, 'false_case'

#define c (c_combo[p])
#define abs_c (c_combo_abs[p])

        for (int p = 0; p < n_c; ++p) {
          if ((c > 0 && row[abs_c - 1] <= 0) || (c < 0 && row[abs_c - 1] > 0)) {
            goto false_case;
          }
        }

#undef c
#undef abs_c

        // if we reached here, we matched all of our conditions
        ++num;

      // false case allows us to skip incrementing num
      false_case : {}
      }

      // insert result into matrix
      output(i, k) = num;
    }
  }

  // set row, column names from the data if possible
  output.attr("dimnames") = List(2);
  if (!Rf_isNull(x.attr("names"))) {
    SET_VECTOR_ELT(output.attr("dimnames"), 0, Rf_duplicate(x.attr("names")));
  }

  if (!Rf_isNull(combos.attr("names"))) {
    SET_VECTOR_ELT(
        output.attr("dimnames"), 1, Rf_duplicate(combos.attr("names")));
  }

  return output;
}

/*** R
## simulate some data
K <- 6 ## number of markers
data <- replicate(10, simplify=FALSE, {
  m <- matrix( rnorm(1E4 * K, 2000, 1000 ), ncol=K )
  m[m < 2500] <- 0
  return(m)
})
names(data) <- sample(letters, 10)

combos <- list(1, 2, 3, 4, 5, 6) ## marginal cell counts
names(combos) <- letters[1:6]
cc <- .CellCounts(data, combos)
f <- function(data) {
  do.call(rbind, lapply(data, function(x) apply(x, 2, function(x) sum(x > 0))))
}
cc2 <- f(data)
colnames(cc2) <- names(combos)
identical(cc, cc2)
library(microbenchmark)
microbenchmark(
  .CellCounts(data, combos),
  f(data)
)
*/

// [[Rcpp::interfaces]]

#include <Rcpp.h>
using namespace Rcpp;

extern "C" SEXP mat2vec(SEXP);

// [[Rcpp::export]]
IntegerMatrix CellCounts_character(List data, List combinations) {

  Function list2env("list2env");

  int m = data.size();          // rows
  int n = combinations.size();  // columns
  IntegerMatrix output(m, n);

  // loop over data
  for (int i = 0; i < m; ++i) {

    SEXP m = PROTECT(mat2vec(as<LogicalMatrix>(data[i])));
    Environment env = list2env(m);

    // loop over combinations
    for (int j = 0; j < n; ++j) {
      ExpressionVector expr = as<ExpressionVector>(combinations[j]);
      SEXP result = PROTECT(Rf_eval(expr[0], env));
      int* result_ptr = INTEGER(result);
      for (int k = 0; k < Rf_length(result); ++k) {
        output(i, j) += result_ptr[k];
      }
      UNPROTECT(1);
    }

    UNPROTECT(1);
  }

  return output;
}

// This file was generated by Rcpp::compileAttributes
// Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

#include <Rcpp.h>

using namespace Rcpp;

// CellCounts_character
IntegerMatrix CellCounts_character(List data, List combinations);
RcppExport SEXP COMPASS_CellCounts_character(SEXP dataSEXP, SEXP combinationsSEXP) {
BEGIN_RCPP
    SEXP __sexp_result;
    {
        Rcpp::RNGScope __rngScope;
        Rcpp::traits::input_parameter< List >::type data(dataSEXP );
        Rcpp::traits::input_parameter< List >::type combinations(combinationsSEXP );
        IntegerMatrix __result = CellCounts_character(data, combinations);
        PROTECT(__sexp_result = Rcpp::wrap(__result));
    }
    UNPROTECT(1);
    return __sexp_result;
END_RCPP
}
// CellCounts
IntegerMatrix CellCounts(List x, List combos);
RcppExport SEXP COMPASS_CellCounts(SEXP xSEXP, SEXP combosSEXP) {
BEGIN_RCPP
    SEXP __sexp_result;
    {
        Rcpp::RNGScope __rngScope;
        Rcpp::traits::input_parameter< List >::type x(xSEXP );
        Rcpp::traits::input_parameter< List >::type combos(combosSEXP );
        IntegerMatrix __result = CellCounts(x, combos);
        PROTECT(__sexp_result = Rcpp::wrap(__result));
    }
    UNPROTECT(1);
    return __sexp_result;
END_RCPP
}

#include <Rcpp.h>
using namespace Rcpp;

// [[register]]
RcppExport SEXP samplePuPs(SEXP alphau,
                           SEXP alphas,
                           SEXP gammat,
                           SEXP T,
                           SEXP K,
                           SEXP nsi,
                           SEXP nui,
                           SEXP d,
                           SEXP M) {
  BEGIN_RCPP

  IntegerVector xnsi(nsi);        // n_s[i,]
  IntegerVector xnui(nui);        // n_u[i,]
  NumericMatrix xalphau(alphau);  // posterior samples for alpha_u
  NumericMatrix xalphas(alphas);  // posterior samples for alpha_s
  IntegerMatrix xgammat(gammat);  // posterior samples for gamma_i
  IntegerMatrix xd(d);            // cytokine combination indicator matrix

  // int xM = as<int>(M); // # markers
  int xT = as<int>(T);  // # MCMC iterations used
  int xK = as<int>(K);  // # Categories
  int K1 = xK - 1;

  NumericVector DIFF(K1);  // Ps-Pu
  NumericVector LogD(K1);  // LogPs-logPu

  int sum = 0;
  std::vector<double> alpha_u(xK);
  std::vector<double> alpha_s(xK);

  NumericMatrix psi(xT, xK);
  NumericMatrix pui(xT, xK);

  std::vector<double> diff(xK);
  std::vector<double> logd(xK);
  double tmp = 0.;
  RNGScope scope;
  double sau = 0.;
  double sas = 0.;
  double sass = 0.;

  for (int tt = 0; tt < xT; tt++) {
    sau = 0.;
    sas = 0.;
    for (int j = 0; j < xK; j++) {
      alpha_u[j] = xalphau(tt, j) + xnui[j];
      alpha_s[j] = xalphas(tt, j) + xnsi[j];
      sau += alpha_u[j];
      sas += alpha_s[j];
    }
    sum = 0;
    for (int j = 0; j < xK; j++) {
      pui(tt, j) = alpha_u[j] / sau;
      sum += xgammat(j, tt);
    }
    if (sum == 0) {
      for (int j = 0; j < xK; j++) {
        psi(tt, j) = pui(tt, j);
        diff[j] = 0;
        logd[j] = 0;
      }
    } else if (sum == xK) {
      for (int j = 0; j < xK; j++) {
        psi(tt, j) = alpha_s[j] / sas;
        diff[j] = psi(tt, j) - pui(tt, j);
        logd[j] = log2(psi(tt, j)) - log2(pui(tt, j));
      }
    } else {
      int l0 = xK - sum;
      std::vector<int> place0(l0);
      std::vector<int> place1(sum);
      int flag0 = 0;
      int flag1 = 0;
      for (int j = 0; j < xK; j++) {
        if (xgammat(j, tt) == 0) {
          place0[flag0] = j;
          flag0 += 1;
        } else {
          place1[flag1] = j;
          flag1 += 1;
        }
      }
      tmp = 0.;
      for (int j = 0; j < l0; j++) {
        psi(tt, place0[j]) = pui(tt, place0[j]);
        tmp += psi(tt, place0[j]);
        diff[place0[j]] = 0;
        logd[place0[j]] = 0;
      }
      std::vector<double> as(sum);
      sass = 0.;
      for (int j = 0; j < sum; j++) {

        as[j] = alpha_s[place1[j]];
        sass += as[j];
      }

      for (int j = 0; j < sum; j++) {
        psi(tt, place1[j]) = as[j] * (1 - tmp) / sass;
        diff[place1[j]] = psi(tt, place1[j]) - pui(tt, place1[j]);
        logd[place1[j]] = log2(psi(tt, place1[j])) - log2(pui(tt, place1[j]));
      }
    }
    for (int j = 0; j < K1; j++) {
      LogD[j] += logd[j] / xT;
      DIFF[j] += diff[j] / xT;
    }
  }

  // compute colmeans psi, pui
  Function colMeans("colMeans");
  NumericVector p_s = colMeans(psi);
  NumericVector p_u = colMeans(pui);

  p_s = p_s[seq(0, p_s.size() - 2)];
  p_u = p_u[seq(0, p_u.size() - 2)];

  return List::create(
      _["p_s"] = p_s, _["p_u"] = p_u, _["diff"] = DIFF, _["logd"] = LogD);

  END_RCPP
}

#include <Rcpp.h>

extern "C" double digamma(double);

// [[register]]
RcppExport SEXP updatealphas_Exp(SEXP alphast,
                                 SEXP n_s,
                                 SEXP K,
                                 SEXP I,
                                 SEXP lambda_s,
                                 SEXP gammat,
                                 SEXP var_1,
                                 SEXP var_2,
                                 SEXP p_var,
                                 SEXP ttt) {
  BEGIN_RCPP
  Rcpp::NumericVector xalphast(alphast);

  Rcpp::IntegerMatrix xn_s(n_s);
  Rcpp::IntegerMatrix xgammat(gammat);
  int xI = Rcpp::as<int>(I);
  int xK = Rcpp::as<int>(K);
  Rcpp::NumericVector sqrt_var1(var_1);
  Rcpp::NumericVector sqrt_var2(var_2);
  int xtt = Rcpp::as<int>(ttt);
  Rcpp::NumericVector xlambda_s(lambda_s);
  Rcpp::IntegerVector xAalphas(xK);

  Rcpp::RNGScope scope;
  Rcpp::NumericVector xp_var(p_var);  // proposal mixture

  double delF = 0.0;
  double psik = 0.;
  double log1 = 0.0;
  double log2 = 0.0;
  double sums = 0.;
  double sum_alp_ns = 0.0;
  double sum_alp = 0.0;
  double sum_gl_alp = 0.0;
  double sum_gl_alp_ns = 0.0;
  int flag1 = 0;
  int flagkk = 0;
  int lp1 = 0;
  for (int kk = 0; kk < xK; kk++) {
    delF = 0.0;
    psik = digamma(xalphast[kk]);
    log1 = 0.0;
    log2 = 0.0;
    for (int i = 0; i < xI; i++) {
      lp1 = 0;
      for (int k = 0; k < xK; k++) {
        if (xgammat(i, k) == 1) {
          lp1 += 1;
        }
      }

      std::vector<int> p1(lp1);
      flag1 = 0;
      flagkk = 0;
      for (int k = 0; k < xK; k++) {
        if (xgammat(i, k) == 1) {
          p1[flag1] = k;
          flag1 += 1;
          if (k == kk) {
            flagkk = 1;
          }
        }
      }
      sum_alp_ns = 0.0;
      sum_alp = 0.0;
      sum_gl_alp = 0.0;
      sum_gl_alp_ns = 0.0;
      for (int k = 0; k < lp1; k++) {
        sums = xalphast[p1[k]] + xn_s(i, p1[k]);
        sum_alp_ns += sums;
        sum_alp += xalphast[p1[k]];
        sum_gl_alp += lgamma(xalphast[p1[k]]);
        sum_gl_alp_ns += lgamma(sums);
      }
      if (flagkk > 0) {
        delF += digamma(xn_s(i, kk) + xalphast[kk]) - psik -
                digamma(sum_alp_ns) + digamma(sum_alp);
      }
      if (lp1 > 0) {
        log2 += -(sum_gl_alp - lgamma(sum_alp)) +
                (sum_gl_alp_ns - lgamma(sum_alp_ns));
      }
    }
    double mean_p = std::max(0.01, xalphast[kk] + delF / xtt);
    Rcpp::NumericVector alpha_s_p = Rcpp::rnorm(1, mean_p, sqrt_var1[kk]);

    if (Rcpp::as<double>(Rcpp::rbinom(1, 1, xp_var[kk])) == 1) {
      alpha_s_p = Rcpp::rnorm(1, mean_p, sqrt_var1[kk]);
    } else {
      alpha_s_p = Rcpp::rnorm(1, mean_p, sqrt_var2[kk]);
    }

    if (alpha_s_p[0] > 0.0) {
      std::vector<double> alp(xK);

      for (int i = 0; i < xK; i++) {
        alp[i] = xalphast[i];
      }
      alp[kk] = alpha_s_p[0];
      // log2 += log(xp_var[kk]*gsl_ran_gaussian_pdf(alp[kk]-mean_p,
      // sqrt_var1[kk])+(1-xp_var[kk])*gsl_ran_gaussian_pdf(alp[kk]-mean_p,
      // sqrt_var2[kk]));
      log2 +=
          log(xp_var[kk] * Rf_dnorm4(alp[kk], mean_p, sqrt_var1[kk], 0) +
              (1 - xp_var[kk]) * Rf_dnorm4(alp[kk], mean_p, sqrt_var2[kk], 0));
      delF = 0.0;
      psik = digamma(alp[kk]);
      for (int i = 0; i < xI; i++) {
        lp1 = 0;
        for (int k = 0; k < xK; k++) {
          if (xgammat(i, k) == 1) {
            lp1 += 1;
          }
        }

        std::vector<int> p1(lp1);
        flag1 = 0;
        flagkk = 0;
        for (int k = 0; k < xK; k++) {
          if (xgammat(i, k) == 1) {
            p1[flag1] = k;
            flag1 += 1;
            if (k == kk) {
              flagkk = 1;
            }
          }
        }

        sum_alp_ns = 0.0;
        sum_alp = 0.0;
        sum_gl_alp = 0.0;
        sum_gl_alp_ns = 0.0;
        for (int k = 0; k < lp1; k++) {
          sums = alp[p1[k]] + xn_s(i, p1[k]);
          sum_alp_ns += sums;
          sum_alp += alp[p1[k]];
          sum_gl_alp += lgamma(alp[p1[k]]);
          sum_gl_alp_ns += lgamma(sums);
        }
        if (flagkk > 0) {
          delF += digamma(xn_s(i, kk) + xalphast[kk]) - psik -
                  digamma(sum_alp_ns) + digamma(sum_alp);
        }
        if (lp1 > 0) {
          log1 += -(sum_gl_alp - lgamma(sum_alp)) +
                  (sum_gl_alp_ns - lgamma(sum_alp_ns));
        }
      }
      mean_p = std::max(0.01, alp[kk] + delF / xtt);
      // log1 +=log(xp_var[kk]*gsl_ran_gaussian_pdf(xalphast[kk]-mean_p,
      // sqrt_var1[kk])+(1-xp_var[kk])*gsl_ran_gaussian_pdf(xalphast[kk]-mean_p,
      // sqrt_var2[kk]));
      log1 += log(
          xp_var[kk] * Rf_dnorm4(xalphast[kk], mean_p, sqrt_var1[kk], 0) +
          (1 - xp_var[kk]) * Rf_dnorm4(xalphast[kk], mean_p, sqrt_var2[kk], 0));

      // log1 += log(gsl_ran_exponential_pdf(alp[kk],xlambda_s[kk]));
      // //exponential prior
      log1 += Rf_dexp(alp[kk], xlambda_s[kk], 1);

      // log2 +=
      // log(gsl_ran_exponential_pdf(xalphast[kk],xlambda_s[kk]));//exponential
      // prior
      log2 += Rf_dexp(xalphast[kk], xlambda_s[kk], 1);

      // if (alp[kk]<0 || alp[kk]>xlambda_s[kk]) {log1+=log(0);} //Uniform prior
      // if (xalphast[kk]<0 || xalphast[kk]>xlambda_s[kk]) {log2+=log(0);}
      // //Uniform prior

      if (log(Rcpp::as<double>(Rcpp::runif(1))) <= (log1 - log2)) {
        xalphast[kk] = alp[kk];
        xAalphas[kk] = 1;
      } else {
        xAalphas[kk] = 0;
      }
    }
  }

  return Rcpp::List::create(Rcpp::Named("alphas_tt") = xalphast,
                            Rcpp::Named("Aalphas") = xAalphas);

  END_RCPP
}

#include <Rcpp.h>

extern "C" double digamma(double);

using namespace std;
using namespace Rcpp;

void updatealphau(vector<double>& xalphaut,
                  vector<int>& xn_s,
                  vector<int>& xn_u,
                  int xI,
                  int xK,
                  vector<double>& xlambda_u,
                  vector<double>& sqrt_var,
                  int xtt,
                  vector<int>& xgammat,
                  vector<int>& xAalphau) {

  double delF = 0.0;
  double log1 = 0.0;
  double log2 = 0.0;
  double sum_alphau = 0.0;
  int flag1 = 0;
  int flag0 = 0;
  int flagkk = 0;
  int temp = 0;
  for (int kk = 0; kk < xK; kk++) {
    delF = 0.0;
    log1 = 0.0;
    log2 = 0.0;
    sum_alphau = 0.0;
    for (int s = 0; s < xK; s++) {
      sum_alphau += xalphaut[s];
    }
    log2 -= xI * lgamma(xalphaut[kk]);
    delF += xI * (digamma(sum_alphau) - digamma(xalphaut[kk]));
    log2 += xI * lgamma(sum_alphau);
    for (int i = 0; i < xI; i++) {
      int lp1 = 0;
      for (int k = 0; k < xK; k++) {
        if (xgammat[xI * k + i] == 1) {
          lp1 += 1;
        }
      }
      int lp0 = xK - lp1;
      vector<int> p1(lp1);
      flag1 = 0;
      vector<int> p0(lp0);
      flag0 = 0;
      flagkk = 0;  // whether gamma_k = 1

      for (int k = 0; k < xK; k++) {
        if (xgammat[xI * k + i] == 1) {
          p1[flag1] = k;
          flag1 += 1;
          if (k == kk) {
            flagkk = 1;
          }
        } else {
          p0[flag0] = k;
          flag0 += 1;
        }
      }
      if (flagkk == 1) {
        log2 += lgamma(xn_u[i + xI * kk] + xalphaut[kk]);
        delF += digamma(xn_u[i + xI * kk] + xalphaut[kk]);
        double sum_nualphau = 0.0;
        double sum_nusalphau = 0.0;
        for (int k = 0; k < lp1; k++) {
          temp = i + xI * p1[k];
          double sum = xn_u[temp] + xalphaut[p1[k]];
          sum_nualphau += sum;
          sum_nusalphau += (sum + xn_s[temp]);
        }
        log2 -= lgamma(sum_nualphau);
        log2 += lgamma(sum_nusalphau + 1);
        delF -= digamma(sum_nualphau);
        delF += digamma(sum_nusalphau + 1);

        for (int k = 0; k < lp0; k++) {
          temp = i + xI * p0[k];
          sum_nusalphau += (xn_u[temp] + xalphaut[p0[k]] + xn_s[temp]);
        }
        delF -= digamma(sum_nusalphau + 1);
        log2 -= lgamma(sum_nusalphau + 1);
      } else {
        log2 += lgamma(xn_u[i + xI * kk] + xalphaut[kk] + xn_s[i + xI * kk]);
        delF += digamma(xn_u[i + xI * kk] + xalphaut[kk] + xn_s[i + kk * xI]);
        double sum_nusalphau = 0.0;
        for (int k = 0; k < xK; k++) {
          sum_nusalphau += xn_u[i + xI * k] + xalphaut[k] + xn_s[i + xI * k];
        }
        log2 -= lgamma(sum_nusalphau + 1);
        delF -= digamma(sum_nusalphau + 1);
      }
    }
    double mean_p = std::max(0.01, xalphaut[kk] + delF / xtt);
    Rcpp::NumericVector alpha_u_p = Rcpp::rnorm(1, mean_p, sqrt_var[kk]);

    if (alpha_u_p[0] > 0.0 && alpha_u_p[0] <= xlambda_u[kk]) {
      vector<double> alp(xK);
      for (int i = 0; i < xK; i++) {
        alp[i] = xalphaut[i];
      }
      alp[kk] = alpha_u_p[0];
      // log2 += log(gsl_ran_gaussian_pdf(alp[kk]-mean_p, sqrt_var[kk]));
      log2 += Rf_dnorm4(alp[kk], mean_p, sqrt_var[kk], 1);

      delF = 0.0;
      sum_alphau = 0.0;
      for (int s = 0; s < xK; s++) {
        sum_alphau += alp[s];
      }
      log1 -= xI * lgamma(alp[kk]);
      delF += xI * (digamma(sum_alphau) - digamma(alp[kk]));
      log1 += xI * lgamma(sum_alphau);
      for (int i = 0; i < xI; i++) {
        int lp1 = 0;
        for (int k = 0; k < xK; k++) {
          if (xgammat[xI * k + i] == 1) {
            lp1 += 1;
          }
        }
        int lp0 = xK - lp1;
        vector<int> p1(lp1);
        flag1 = 0;
        vector<int> p0(lp0);
        flag0 = 0;
        flagkk = 0;  // whether gamma_k = 1

        for (int k = 0; k < xK; k++) {
          if (xgammat[xI * k + i] == 1) {
            p1[flag1] = k;
            flag1 += 1;
            if (k == kk) {
              flagkk = 1;
            }
          } else {
            p0[flag0] = k;
            flag0 += 1;
          }
        }
        if (flagkk == 1) {
          log1 += lgamma(xn_u[i + xI * kk] + alp[kk]);
          delF += digamma(xn_u[i + xI * kk] + alp[kk]);
          double sum_nualphau = 0.0;
          double sum_nusalphau = 0.0;
          for (int k = 0; k < lp1; k++) {
            temp = i + xI * p1[k];
            double sum = xn_u[temp] + alp[p1[k]];
            sum_nualphau += sum;
            sum_nusalphau += (sum + xn_s[temp]);
          }
          log1 -= lgamma(sum_nualphau);
          log1 += lgamma(sum_nusalphau + 1);
          delF -= digamma(sum_nualphau);
          delF += digamma(sum_nusalphau + 1);

          for (int k = 0; k < lp0; k++) {
            sum_nusalphau +=
                (xn_u[i + xI * p0[k]] + alp[p0[k]] + xn_s[i + xI * p0[k]]);
          }
          delF -= digamma(sum_nusalphau + 1);
          log1 -= lgamma(sum_nusalphau + 1);
        } else {
          log1 += lgamma(xn_u[i + xI * kk] + alp[kk] + xn_s[i + xI * kk]);
          delF += digamma(xn_u[i + xI * kk] + alp[kk] + xn_s[i + xI * kk]);
          double sum_nusalphau = 0.0;
          for (int k = 0; k < xK; k++) {
            temp = i + xI * k;
            sum_nusalphau += xn_u[temp] + alp[k] + xn_s[temp];
          }
          log1 -= lgamma(sum_nusalphau + 1);
          delF -= digamma(sum_nusalphau + 1);
        }
      }
      mean_p = std::max(0.01, alp[kk] + delF / xtt);
      // log1 +=log(gsl_ran_gaussian_pdf(xalphaut[kk]-mean_p, sqrt_var[kk]));
      log1 += Rf_dnorm4(xalphaut[kk], mean_p, sqrt_var[kk], 1);

      if (log(Rcpp::as<double>(Rcpp::runif(1))) <= (log1 - log2)) {
        xalphaut[kk] = alp[kk];
        xAalphau[kk] = 1;
      }
    }
  }
}

#include <Rcpp.h>
extern "C" double digamma(double);

// [[register]]
RcppExport SEXP updatealphau_noPu_Exp(SEXP alphaut,
                                      SEXP n_s,
                                      SEXP n_u,
                                      SEXP I,
                                      SEXP K,
                                      SEXP lambda_u,
                                      SEXP var_p,
                                      SEXP ttt,
                                      SEXP gammat) {
  BEGIN_RCPP
  Rcpp::IntegerMatrix xgammat(gammat);
  Rcpp::NumericVector xalphaut(alphaut);
  Rcpp::IntegerMatrix xn_s(n_s);
  Rcpp::IntegerMatrix xn_u(n_u);
  int xI = Rcpp::as<int>(I);
  int xK = Rcpp::as<int>(K);
  Rcpp::NumericVector sqrt_var(var_p);
  int xtt = Rcpp::as<int>(ttt);
  Rcpp::NumericVector xlambda_u(lambda_u);
  Rcpp::IntegerVector xAalphau(xK);

  Rcpp::RNGScope scope;

  double delF = 0.0;
  double log1 = 0.0;
  double log2 = 0.0;
  double sum_alphau = 0.0;
  int flag1 = 0;
  int flag0 = 0;
  int flagkk = 0;
  int lp0 = 0;
  int lp1 = 0;
  double sum_nusalphau = 0.0;
  double sum_nualphau = 0.0;
  double sums = 0.;
  for (int kk = 0; kk < xK; kk++) {
    delF = 0.0;
    log1 = 0.0;
    log2 = 0.0;
    sum_alphau = 0.0;
    for (int s = 0; s < xK; s++) {
      sum_alphau += xalphaut[s];
    }
    log2 -= xI * lgamma(xalphaut[kk]);
    delF += xI * (digamma(sum_alphau) - digamma(xalphaut[kk]));
    log2 += xI * lgamma(sum_alphau);
    for (int i = 0; i < xI; i++) {
      lp1 = 0;
      for (int k = 0; k < xK; k++) {
        if (xgammat(i, k) == 1) {
          lp1 += 1;
        }
      }
      lp0 = xK - lp1;
      std::vector<int> p1(lp1);
      flag1 = 0;
      std::vector<int> p0(lp0);
      flag0 = 0;
      flagkk = 0;  // whether gamma_k = 1

      for (int k = 0; k < xK; k++) {
        if (xgammat(i, k) == 1) {
          p1[flag1] = k;
          flag1 += 1;
          if (k == kk) {
            flagkk = 1;
          }
        } else {
          p0[flag0] = k;
          flag0 += 1;
        }
      }
      if (flagkk == 1) {
        log2 += lgamma(xn_u(i, kk) + xalphaut[kk]);
        delF += digamma(xn_u(i, kk) + xalphaut[kk]);
        sum_nualphau = 0.0;
        sum_nusalphau = 0.0;
        for (int k = 0; k < lp1; k++) {
          sums = xn_u(i, p1[k]) + xalphaut[p1[k]];
          sum_nualphau += sums;
          sum_nusalphau += (sums + xn_s(i, p1[k]));
        }
        log2 -= lgamma(sum_nualphau);
        log2 += lgamma(sum_nusalphau + 1);
        delF -= digamma(sum_nualphau);
        delF += digamma(sum_nusalphau + 1);

        for (int k = 0; k < lp0; k++) {
          sum_nusalphau += (xn_u(i, p0[k]) + xalphaut[p0[k]] + xn_s(i, p0[k]));
        }
        delF -= digamma(sum_nusalphau + 1);
        log2 -= lgamma(sum_nusalphau + 1);
      } else {
        log2 += lgamma(xn_u(i, kk) + xalphaut[kk] + xn_s(i, kk));
        delF += digamma(xn_u(i, kk) + xalphaut[kk] + xn_s(i, kk));
        sum_nusalphau = 0.0;
        for (int k = 0; k < xK; k++) {
          sum_nusalphau += xn_u(i, k) + xalphaut[k] + xn_s(i, k);
        }
        log2 -= lgamma(sum_nusalphau + 1);
        delF -= digamma(sum_nusalphau + 1);
      }
    }
    double mean_p = std::max(0.01, xalphaut[kk] + delF / xtt);
    Rcpp::NumericVector alpha_u_p = Rcpp::rnorm(1, mean_p, sqrt_var[kk]);
    if (alpha_u_p[0] > 0.0) {
      std::vector<double> alp(xK);
      for (int i = 0; i < xK; i++) {
        alp[i] = xalphaut[i];
      }
      alp[kk] = alpha_u_p[0];

      // log2 += log(gsl_ran_gaussian_pdf(alp[kk]-mean_p, sqrt_var[kk]));
      log2 += Rf_dnorm4(alp[kk], mean_p, sqrt_var[kk], 1);

      delF = 0.0;
      sum_alphau = 0.0;
      for (int s = 0; s < xK; s++) {
        sum_alphau += alp[s];
      }
      log1 -= xI * lgamma(alp[kk]);
      delF += xI * (digamma(sum_alphau) - digamma(alp[kk]));
      log1 += xI * lgamma(sum_alphau);
      for (int i = 0; i < xI; i++) {
        lp1 = 0;
        for (int k = 0; k < xK; k++) {
          if (xgammat(i, k) == 1) {
            lp1 += 1;
          }
        }
        lp0 = xK - lp1;
        std::vector<int> p1(lp1);
        flag1 = 0;
        std::vector<int> p0(lp0);
        flag0 = 0;
        flagkk = 0;  // whether gamma_k = 1

        for (int k = 0; k < xK; k++) {
          if (xgammat(i, k) == 1) {
            p1[flag1] = k;
            flag1 += 1;
            if (k == kk) {
              flagkk = 1;
            }
          } else {
            p0[flag0] = k;
            flag0 += 1;
          }
        }
        if (flagkk == 1) {
          log1 += lgamma(xn_u(i, kk) + alp[kk]);
          delF += digamma(xn_u(i, kk) + alp[kk]);
          sum_nualphau = 0.0;
          sum_nusalphau = 0.0;
          for (int k = 0; k < lp1; k++) {
            sums = xn_u(i, p1[k]) + alp[p1[k]];
            sum_nualphau += sums;
            sum_nusalphau += (sums + xn_s(i, p1[k]));
          }
          log1 -= lgamma(sum_nualphau);
          log1 += lgamma(sum_nusalphau + 1);
          delF -= digamma(sum_nualphau);
          delF += digamma(sum_nusalphau + 1);

          for (int k = 0; k < lp0; k++) {
            sum_nusalphau += (xn_u(i, p0[k]) + alp[p0[k]] + xn_s(i, p0[k]));
          }
          delF -= digamma(sum_nusalphau + 1);
          log1 -= lgamma(sum_nusalphau + 1);
        } else {
          log1 += lgamma(xn_u(i, kk) + alp[kk] + xn_s(i, kk));
          delF += digamma(xn_u(i, kk) + alp[kk] + xn_s(i, kk));
          sum_nusalphau = 0.0;
          for (int k = 0; k < xK; k++) {
            sum_nusalphau += xn_u(i, k) + alp[k] + xn_s(i, k);
          }
          log1 -= lgamma(sum_nusalphau + 1);
          delF -= digamma(sum_nusalphau + 1);
        }
      }
      mean_p = std::max(0.01, alp[kk] + delF / xtt);

      // log1 +=log(gsl_ran_gaussian_pdf(xalphaut[kk]-mean_p, sqrt_var[kk]));
      log1 += Rf_dnorm4(xalphaut[kk], mean_p, sqrt_var[kk], 1);

      // log1 += log(gsl_ran_exponential_pdf(alp[kk],xlambda_u[kk]));
      // //exponential prior
      log1 += Rf_dexp(alp[kk], xlambda_u[kk], 1);

      // log2 += log(gsl_ran_exponential_pdf(xalphaut[kk],xlambda_u[kk]));
      // //exponential prior
      log2 += Rf_dexp(xalphaut[kk], xlambda_u[kk], 1);

      // if (alp[kk]<0 || alp[kk]>xlambda_u[kk]) {log1+=log(0);} //Uniform prior
      // if (xalphaut[kk]<0 || xalphaut[kk]>xlambda_u[kk]) {log2+=log(0);}
      // //Uniform prior

      if (log(Rcpp::as<double>(Rcpp::runif(1))) <= (log1 - log2)) {
        xalphaut[kk] = alp[kk];
        xAalphau[kk] = 1;
      } else {
        xAalphau[kk] = 0;
      }
    }
  }

  return Rcpp::List::create(Rcpp::Named("alphau_tt") = xalphaut,
                            Rcpp::Named("Aalphau") = xAalphau);

  END_RCPP
}

// update beta with Random walk proposal
#include <Rcpp.h>
using namespace std;

void updatebeta_RW(vector<double>& xmust,
                   vector<double>& xmuut,
                   vector<int>& xn_s,
                   vector<int>& xn_u,
                   int xI,
                   int xK,
                   int xM,
                   int K1,
                   int t1,
                   double xp_var,
                   double sqrt_var1,
                   double sqrt_var2,
                   int xtt,
                   vector<int>& xgammat,
                   Rcpp::IntegerMatrix& xd,
                   Rcpp::NumericMatrix& xybar_s,
                   Rcpp::NumericMatrix& xybar_u,
                   Rcpp::NumericMatrix& pt1,
                   Rcpp::NumericMatrix& pt2,
                   Rcpp::NumericMatrix& pt3,
                   double xlambda,
                   Rcpp::NumericVector& beta,
                   double xalpha,
                   double xsig_beta1,
                   double xbeta1,
                   Rcpp::IntegerVector& xAbeta) {

  double log1 = 0.;
  double log2 = 0.;
  int nik = 0;
  int temp = 0;
  double beta_np = 0.;
  double alpha_np = 0.;
  double xbeta = beta[t1];
  int tt = t1 + 1;
  beta[tt] = xbeta;

  Rcpp::NumericVector betap = Rcpp::rnorm(1, xbeta, sqrt_var1);
  if (Rcpp::as<double>(Rcpp::rbinom(1, 1, xp_var)) == 1) {
    betap = Rcpp::rnorm(1, xbeta, sqrt_var1);
  } else {
    betap = Rcpp::rnorm(1, xbeta, sqrt_var2);
  }

  if (betap[0] > 0.0) {
    double alogb = xalpha * log(xbeta);
    double alogb0 = xalpha * log(betap[0]);
    for (int p = 0; p < xM; p++) {

      for (int i = 0; i < xI; i++) {
        for (int k = 0; k < K1; k++) {
          temp = xI * k + i;
          if (xd(k, p) == 1) {
            nik = xn_u[temp] + xn_s[temp];
            if (xgammat[temp] == 1) {
              if (xn_u[temp] > 0 && xn_s[temp] > 0) {
                beta_np = log(pt3(temp, p) + xbeta +
                              pow((xmuut[p] - xybar_u(temp, p)), 2) /
                                  (2 * (xlambda + 1 / xn_u[temp])) +
                              pow((xmust[p] - xybar_s(temp, p)), 2) /
                                  (2 * (xlambda + 1 / xn_s[temp])));
                alpha_np = nik / 2 + xalpha;
                log2 += alogb - alpha_np * beta_np;
                beta_np = log(pt3(temp, p) + betap[0] +
                              pow((xmuut[p] - xybar_u(temp, p)), 2) /
                                  (2 * (xlambda + 1 / xn_u[temp])) +
                              pow((xmust[p] - xybar_s(temp, p)), 2) /
                                  (2 * (xlambda + 1 / xn_s[temp])));
                log1 += alogb0 - alpha_np * beta_np;

              } else if (xn_u[temp] == 0 && xn_s[temp] > 0) {
                beta_np = log(pt1(temp, p) + xbeta +
                              pow((xmust[p] - xybar_s(temp, p)), 2) /
                                  (2 * (xlambda + 1 / xn_s[temp])));
                alpha_np = nik / 2 + xalpha;
                log2 += alogb - alpha_np * beta_np;
                beta_np = log(pt1(temp, p) + betap[0] +
                              pow((xmust[p] - xybar_s(temp, p)), 2) /
                                  (2 * (xlambda + 1 / xn_s[temp])));
                log1 += alogb0 - alpha_np * beta_np;

              } else if (xn_u[temp] > 0 && xn_s[temp] == 0) {
                beta_np = log(pt2(temp, p) + xbeta +
                              pow((xmuut[p] - xybar_u(temp, p)), 2) /
                                  (2 * (xlambda + 1 / xn_u[temp])));
                alpha_np = nik / 2 + xalpha;
                log2 += alogb - alpha_np * beta_np;
                beta_np = log(pt2(temp, p) + betap[0] +
                              pow((xmuut[p] - xybar_u(temp, p)), 2) /
                                  (2 * (xlambda + 1 / xn_u[temp])));
                log1 += alogb0 - alpha_np * beta_np;
              }
            } else {
              if (nik > 0) {
                beta_np = log(pt3(temp, p) + xbeta +
                              pow((xmuut[p] - (xn_s[temp] * xybar_s(temp, p) +
                                               xn_u[temp] * xybar_u(temp, p)) /
                                                  nik),
                                  2) /
                                  (2 * (xlambda + 1 / nik)) +
                              0.5 * xn_s[temp] * pow(xybar_s(temp, p), 2) +
                              0.5 * xn_u[temp] * pow(xybar_u(temp, p), 2) -
                              0.5 * pow((xn_s[temp] * xybar_s(temp, p) +
                                         xn_u[temp] * xybar_u(temp, p)),
                                        2) /
                                  nik);
                alpha_np = nik / 2 + xalpha;
                log2 += alogb - alpha_np * beta_np;
                beta_np = log(pt3(temp, p) + betap[0] +
                              pow((xmuut[p] - (xn_s[temp] * xybar_s(temp, p) +
                                               xn_u[temp] * xybar_u(temp, p)) /
                                                  nik),
                                  2) /
                                  (2 * (xlambda + 1 / nik)) +
                              0.5 * xn_s[temp] * pow(xybar_s(temp, p), 2) +
                              0.5 * xn_u[temp] * pow(xybar_u(temp, p), 2) -
                              0.5 * pow((xn_s[temp] * xybar_s(temp, p) +
                                         xn_u[temp] * xybar_u(temp, p)),
                                        2) /
                                  nik);
                log1 += alogb0 - alpha_np * beta_np;
              }
            }
          }
        }
      }
    }

    // log2+=log(gsl_ran_gaussian_pdf(xbeta-xbeta1, xsig_beta1));
    log2 += Rf_dnorm4(xbeta, xbeta1, xsig_beta1, 1);

    // log1+=log(gsl_ran_gaussian_pdf(betap[0]-xbeta1, xsig_beta1));
    log1 += Rf_dnorm4(betap[0], xbeta1, xsig_beta1, 1);

    if (log(Rcpp::as<double>(Rcpp::runif(1))) <= (log1 - log2)) {
      beta[tt] = betap[0];
      xAbeta[tt] = 1;
    }
  }
}

#include <Rcpp.h>

#include <R_ext/Utils.h>
#include <stdlib.h>

// [[register]]
RcppExport SEXP updategammak_noPu(SEXP n_s,
                                  SEXP n_u,
                                  SEXP gammat,
                                  SEXP I,
                                  SEXP K,
                                  SEXP SS,
                                  SEXP alphau,
                                  SEXP alphas,
                                  SEXP alpha,
                                  SEXP mk,
                                  SEXP Istar,
                                  SEXP mKstar,
                                  SEXP pp,
                                  SEXP pb1,
                                  SEXP pb2,
                                  SEXP indi) {
  BEGIN_RCPP

  Rcpp::IntegerMatrix xn_s(n_s);
  Rcpp::IntegerMatrix xn_u(n_u);
  Rcpp::NumericVector xalphas(alphas);
  Rcpp::IntegerMatrix xindicator(indi);
  Rcpp::NumericVector xalphau(alphau);
  double xalpha = Rcpp::as<double>(alpha);
  int xK = Rcpp::as<int>(K);
  int xI = Rcpp::as<int>(I);
  int xSS = Rcpp::as<int>(SS);
  Rcpp::IntegerVector xAg(xI);
  Rcpp::IntegerVector gamma_old(xK);
  Rcpp::IntegerMatrix gamma_tt(gammat);
  Rcpp::IntegerVector xmk(mk);
  int xIstar = Rcpp::as<int>(Istar);
  int xmKstar = Rcpp::as<int>(mKstar);
  Rcpp::NumericVector xpp(pp);
  double xpb1 = Rcpp::as<double>(pb1);
  double xpb2 = Rcpp::as<double>(pb2);
  Rcpp::RNGScope scope;
  Rcpp::NumericVector m(1);
  int K1 = xK - 1;
  std::vector<int> array1K(K1);
  std::vector<int> mik(K1);
  double alphaK = xalpha / xK;
  int xindiK1 = 0;

  int miKstar = 0;
  int Iistar = 0;
  int flag = 0;
  int flag_old = 0;
  double log_likeli_deno = 0.;
  double log_likeli_nume = 0.0;
  int ct = 0;
  double log_probK = 0.;
  double log_1probK = 0.;
  int vec = 0;
  int arr = 0;
  int lplace0_new = 0;
  int lplace0 = 0;
  int flag_prop = 0;
  int lplace1_new = 0;
  int lplace1 = 0;
  for (int i = 0; i < xI; i++) {
    xindiK1 = xindicator(i, K1);
    std::vector<int> array1K(xindiK1);
    for (int j = 0; j < xK; j++) {
      gamma_old[j] = gamma_tt(i, j);
    }
    for (int ss = 0; ss < xSS; ss++) {
      if (xindiK1 > 0) {
        miKstar = xmKstar;
        Iistar = xIstar;
        flag = 0;
        flag_old = 0;
        log_likeli_deno = 0.;
        ct = 0;
        for (int j = 0; j < K1; j++) {
          gamma_tt(i, j) = gamma_old[j];
          mik[j] = xmk[j] - gamma_old[j];
          flag += gamma_tt(i, j);
          if (xindicator(i, j) == 1) {
            array1K[ct] = j;
            ct += 1;
          }
          log_likeli_deno +=
              gamma_old[j] * (log(mik[j] + alphaK) - log(xI + alphaK)) +
              (1 - gamma_old[j]) * (log(xI - mik[j]) - log(xI + alphaK));
        }
        if (flag > 1) {  //*
          Iistar = xIstar - 1;
          miKstar = xmKstar - gamma_old[K1];
          flag_old = 1;
        }
        gamma_tt(i, K1) = gamma_old[K1];
        log_probK = log(miKstar + alphaK) - log(Iistar + alphaK + 1);
        log_1probK = log(Iistar - miKstar + 1) - log(Iistar + alphaK + 1);
        if (flag_old == 1) {
          log_likeli_deno +=
              gamma_old[K1] * (log_probK) + (1 - gamma_old[K1]) * (log_1probK);
        }
        if (xindiK1 <= 2) {
          m = 1;
        } else {
          if (Rcpp::as<double>(Rcpp::runif(1)) <= xpp[i]) {
            m = Rcpp::rbinom(1, xindiK1, xpb1);
          }  // m[0] +=1;}
          else {
            m = Rcpp::rbinom(1, xindiK1, xpb2);
          }  // m[0] +=1;}
        }
        /* random sample */
        if (m[0] > 0) {
          for (int k = 0; k < m[0]; k++) {
            Rcpp::NumericVector tmp = Rcpp::runif(1, k, xindiK1);
            vec = (int)tmp[0];
            arr = array1K[k];
            array1K[k] = array1K[vec];
            array1K[vec] = arr;
            gamma_tt(i, array1K[k]) = 1 - gamma_tt(i, array1K[k]);
            // if (Rcpp::as<double>(Rcpp::runif(1)) < 0.5)
            // {gamma_tt(i,array1K[k]) = 1;}
            // else {gamma_tt(i,array1K[k]) = 0;}
          }
        }
        lplace0_new = 0;
        lplace0 = 0;
        log_likeli_nume = 0.0;
        for (int k = 0; k < K1; k++) {
          log_likeli_nume +=
              gamma_tt(i, k) * (log(mik[k] + alphaK) - log(xI + alphaK)) +
              (1 - gamma_tt(i, k)) * (log(xI - mik[k]) - log(xI + alphaK));
          if (gamma_tt(i, k) == 0) {
            lplace0_new += 1;
          }
          if (gamma_old[k] == 0) {
            lplace0 += 1;
          }
        }
        lplace1_new = K1 - lplace0_new;
        lplace1 = K1 - lplace0;
        flag_prop = 0;
        if (lplace1_new == 1) {  //*
          gamma_tt(i, K1) = 1;
          lplace1_new += 1;
        } else if (lplace1_new == 0) {
          gamma_tt(i, K1) = 0;
          lplace0_new += 1;
        } else {
          flag_prop = 1;
          log_likeli_deno += log(0.5);  // proposal
          if (Rcpp::as<double>(Rcpp::runif(1)) <= 0.5) {
            gamma_tt(i, K1) = 1;
          } else {
            gamma_tt(i, K1) = 0;
          }
          if (gamma_tt(i, K1) == 1) {
            lplace1_new += 1;
            log_likeli_nume += log_probK;
          } else {
            lplace0_new += 1;
            log_likeli_nume += log_1probK;
          }
        }
        if (lplace1 == 1) {  //*
          lplace1 += 1;
        } else if (lplace1 == 0) {
          lplace0 += 1;
        } else {
          log_likeli_nume += log(0.5);  // proposal
          if (gamma_old[K1] == 1) {
            lplace1 += 1;
          } else {
            lplace0 += 1;
          }
        }

        std::vector<int> place0_new(lplace0_new);
        std::vector<int> place0(lplace0);
        std::vector<int> place1(lplace1);
        int flag0 = 0;
        int flag1 = 0;
        int flag0_old = 0;
        int flag1_old = 0;
        std::vector<int> place1_new(lplace1_new);
        for (int k = 0; k < xK; k++) {
          if (gamma_tt(i, k) == 0) {
            place0_new[flag0] = k;
            flag0 += 1;
          } else {
            place1_new[flag1] = k;
            flag1 += 1;
          }
          if (gamma_old[k] == 0) {
            place0[flag0_old] = k;
            flag0_old += 1;
          } else {
            place1[flag1_old] = k;
            flag1_old += 1;
          }
        }

        if (lplace1_new == 0) {
          double sum_alphau_nus = 0.0;
          double sum_gammaln_nu = 0.0;

          for (int k = 0; k < xK; k++) {
            double summ = xalphau[k] + xn_u(i, k) + xn_s(i, k);
            sum_alphau_nus += summ;
            sum_gammaln_nu += lgamma(summ);
          }
          log_likeli_nume += (sum_gammaln_nu - lgamma(sum_alphau_nus));

        } else if (lplace0_new == 0) {
          double sum_alphau_nus = 0.0;
          double sum_gammaln_nu = 0.0;
          double sum_gammaln = 0.0;
          double sum_alphas = 0.0;
          double sum_gammaln_ns = 0.0;
          double sum_alphas_ns = 0.0;

          for (int k = 0; k < xK; k++) {
            double summ = xalphau[k] + xn_u(i, k);
            sum_alphau_nus += summ;
            sum_gammaln_nu += lgamma(summ);

            sum_gammaln += lgamma(xalphas[k]);
            sum_alphas += xalphas[k];
            sum_gammaln_ns += (lgamma(xalphas[k] + xn_s(i, k)));
            sum_alphas_ns += (xalphas[k] + xn_s(i, k));
          }
          log_likeli_nume += (sum_gammaln_nu - lgamma(sum_alphau_nus));
          log_likeli_nume -= (sum_gammaln - lgamma(sum_alphas));
          log_likeli_nume += (sum_gammaln_ns - lgamma(sum_alphas_ns));
        } else {
          double sum_gammaln = 0.0;
          double sum_alphas = 0.0;
          double sum_gammaln_ns = 0.0;
          double sum_alphas_ns = 0.0;
          double sum_alphau_nu = 0.0;
          double sum_alphau_nus = 0.0;
          double sum_gammaln_nu = 0.0;
          for (int k = 0; k < lplace1_new; k++) {
            sum_alphau_nu += (xalphau[place1_new[k]] + xn_u(i, place1_new[k]));
            sum_alphau_nus += (xalphau[place1_new[k]] + xn_u(i, place1_new[k]) +
                               xn_s(i, place1_new[k]));
            sum_gammaln_nu +=
                lgamma(xalphau[place1_new[k]] + xn_u(i, place1_new[k]));
            sum_gammaln += lgamma(xalphas[place1_new[k]]);
            sum_alphas += xalphas[place1_new[k]];
            sum_gammaln_ns +=
                (lgamma(xalphas[place1_new[k]] + xn_s(i, place1_new[k])));
            sum_alphas_ns += (xalphas[place1_new[k]] + xn_s(i, place1_new[k]));
          }
          log_likeli_nume -= (sum_gammaln - lgamma(sum_alphas));
          log_likeli_nume += (sum_gammaln_ns - lgamma(sum_alphas_ns));
          log_likeli_nume += (sum_gammaln_nu - lgamma(sum_alphau_nu));

          log_likeli_nume += lgamma(sum_alphau_nus + 1);
          sum_gammaln_nu = 0.0;
          for (int k = 0; k < lplace0_new; k++) {
            sum_alphau_nus += (xalphau[place0_new[k]] + xn_u(i, place0_new[k]) +
                               xn_s(i, place0_new[k]));
            sum_gammaln_nu +=
                lgamma(xalphau[place0_new[k]] + xn_u(i, place0_new[k]) +
                       xn_s(i, place0_new[k]));
          }
          log_likeli_nume += sum_gammaln_nu;
          log_likeli_nume -= lgamma(sum_alphau_nus + 1);
        }

        //
        if (lplace1 == 0) {
          double sum_alphau_nus = 0.0;
          double sum_gammaln_nu = 0.0;
          for (int k = 0; k < xK; k++) {
            double summ = xalphau[k] + xn_u(i, k) + xn_s(i, k);
            sum_alphau_nus += summ;
            sum_gammaln_nu += lgamma(summ);
          }
          log_likeli_deno += (sum_gammaln_nu - lgamma(sum_alphau_nus));
        } else if (lplace0 == 0) {
          double sum_alphau_nus = 0.0;
          double sum_gammaln_nu = 0.0;
          double sum_gammaln = 0.0;
          double sum_alphas = 0.0;
          double sum_gammaln_ns = 0.0;
          double sum_alphas_ns = 0.0;

          for (int k = 0; k < xK; k++) {
            double summ = xalphau[k] + xn_u(i, k);
            sum_alphau_nus += summ;
            sum_gammaln_nu += lgamma(summ);

            sum_gammaln += lgamma(xalphas[k]);
            sum_alphas += xalphas[k];
            sum_gammaln_ns += (lgamma(xalphas[k] + xn_s(i, k)));
            sum_alphas_ns += (xalphas[k] + xn_s(i, k));
          }
          log_likeli_deno += (sum_gammaln_nu - lgamma(sum_alphau_nus));
          log_likeli_deno -= (sum_gammaln - lgamma(sum_alphas));
          log_likeli_deno += (sum_gammaln_ns - lgamma(sum_alphas_ns));
        } else {
          double sum_gammaln = 0.0;
          double sum_alphas = 0.0;
          double sum_gammaln_ns = 0.0;
          double sum_alphas_ns = 0.0;
          double sum_alphau_nu = 0.0;
          double sum_alphau_nus = 0.0;
          double sum_gammaln_nu = 0.0;
          for (int k = 0; k < lplace1; k++) {
            sum_alphau_nu += (xalphau[place1[k]] + xn_u(i, place1[k]));
            sum_alphau_nus +=
                (xalphau[place1[k]] + xn_u(i, place1[k]) + xn_s(i, place1[k]));
            sum_gammaln_nu += lgamma(xalphau[place1[k]] + xn_u(i, place1[k]));
            sum_gammaln += lgamma(xalphas[place1[k]]);
            sum_alphas += xalphas[place1[k]];
            sum_gammaln_ns += (lgamma(xalphas[place1[k]] + xn_s(i, place1[k])));
            sum_alphas_ns += (xalphas[place1[k]] + xn_s(i, place1[k]));
          }
          log_likeli_deno -= (sum_gammaln - lgamma(sum_alphas));
          log_likeli_deno += (sum_gammaln_ns - lgamma(sum_alphas_ns));
          log_likeli_deno += (sum_gammaln_nu - lgamma(sum_alphau_nu));

          log_likeli_deno += lgamma(sum_alphau_nus + 1);
          sum_gammaln_nu = 0.0;
          for (int k = 0; k < lplace0; k++) {
            sum_alphau_nus +=
                (xalphau[place0[k]] + xn_u(i, place0[k]) + xn_s(i, place0[k]));
            sum_gammaln_nu += lgamma(xalphau[place0[k]] + xn_u(i, place0[k]) +
                                     xn_s(i, place0[k]));
          }
          log_likeli_deno += sum_gammaln_nu;
          log_likeli_deno -= lgamma(sum_alphau_nus + 1);
        }

        // std::cout <<"log_nume = "<< log_likeli_nume << std::endl;
        // std::cout <<"log_deo = "<< log_likeli_deno << std::endl;
        if (log(Rcpp::as<double>(Rcpp::runif(1))) <=
            (log_likeli_nume - log_likeli_deno)) {
          for (int j = 0; j < K1; j++) {
            xmk[j] = xmk[j] - gamma_old[j] + gamma_tt(i, j);
            gamma_old[j] = gamma_tt(i, j);
          }
          gamma_old[K1] = gamma_tt(i, K1);
          xIstar = Iistar + flag_prop;
          xmKstar = miKstar + flag_prop * gamma_tt(i, K1);
          xAg[i] = 1;
        } else {
          for (int j = 0; j < xK; j++) {
            gamma_tt(i, j) = gamma_old[j];
          }
        }
      }
    }
  }

  return Rcpp::List::create(Rcpp::Named("gamma_tt") = gamma_tt,
                            Rcpp::Named("Ag") = xAg,
                            Rcpp::Named("mKstar") = xmKstar,
                            Rcpp::Named("xmk") = xmk,
                            Rcpp::Named("xIstar") = xIstar);

  END_RCPP
}

